# Generated by Django 3.2.7 on 2022-03-10 11:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0005_auto_20220303_0958'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='reportquery',
            options={'ordering': ['order'], 'verbose_name_plural': 'Report queries'},
        ),
        migrations.AddField(
            model_name='reportquery',
            name='order',
            field=models.PositiveSmallIntegerField(default=1),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='reportquery',
            name='name',
            field=models.CharField(default='Standard', max_length=256),
        ),
        migrations.AlterUniqueTogether(
            name='reportquery',
            unique_together=set(),
        ),
    ]
